#ifndef FGGL_GFX_OGL_COMPAT_H
#define FGGL_GFX_OGL_COMPAT_H
/**
 * Legacy/Direct OpenGL calls.
 *
 * This shouldn't be exposed to the demo app, but the ECS we're using isn't smart enouph to allow us to
 * abstract this yet. It's next thing on the list, but this branch is about cleaning up OpenGL not about
 * extending our ECS.
 *
 * Should be removed when the engine has suitable abstractions in place.
 */

#include <functional>

#include <fggl/gfx/ogl/shader.hpp>
#include <fggl/gfx/ogl/renderer.hpp>

#include <fggl/gfx/common.hpp>
#include <fggl/gfx/camera.hpp>
#include <fggl/ecs/ecs.hpp>
#include <utility>
#include <fggl/input/camera_input.h>
#include <fggl/data/heightmap.h>

namespace fggl::gfx {

    void generateHeightMesh(data::HeightMap* heightMap, data::Mesh);

	//
	// fake module support - allows us to still RAII
	//
	struct ecsOpenGLModule : ecs3::Module {
		fggl::gfx::MeshRenderer renderer;
		fggl::gfx::ShaderCache cache;

		ecsOpenGLModule(Window& window, std::shared_ptr<fggl::data::Storage> storage) :
            renderer(),
            cache(std::move(storage)) {	}

        std::string name() const override {
            return "gfx::opengl";
        }

        void uploadMesh(ecs3::World* world, ecs::entity_t entity) {
            auto meshData = world->get<gfx::StaticMesh>(entity);

            auto pipeline = cache.get(meshData->pipeline);
            auto glMesh = renderer.upload(meshData->mesh);

            glMesh.pipeline = pipeline;
            world->set<fggl::gfx::GlRenderToken>(entity, &glMesh);
        }

        void uploadHeightmap(ecs3::World *world, ecs::entity_t entity) {
            const auto heightmap = world->get<data::HeightMap>(entity);

            data::Mesh tmpMesh{};
            data::generateHeightMesh(heightmap, tmpMesh);
            auto glMesh = renderer.upload( tmpMesh );

            auto pipeline = cache.get("phong");
            glMesh.pipeline = pipeline;
            glMesh.renderType = GlRenderType::triangle_strip;
            world->set<fggl::gfx::GlRenderToken>(entity, &glMesh);
        }

        void onLoad(ecs3::ModuleManager& manager, ecs3::TypeRegistry& types) override {
            // TODO implement dependencies
            types.make<fggl::gfx::StaticMesh>();
            types.make<fggl::data::HeightMap>();
            types.make<fggl::gfx::Camera>();

            // FIXME probably shouldn't be doing this...
            types.make<fggl::input::FreeCamKeys>();

            // opengl
            types.make<fggl::gfx::GlRenderToken>();

            // callbacks
            auto upload_cb = [this](auto a, auto b) { this->uploadMesh(a, b); };
            manager.onAdd<fggl::gfx::StaticMesh>( upload_cb );
            manager.onAdd<fggl::data::HeightMap>( [this](auto a, auto b) { this->uploadHeightmap(a, b); });
        }

	};
	using OglModule = std::shared_ptr<ecsOpenGLModule>;

	//
	// Loading related functions - should be handled in modules/data-driven
	//
	inline void loadPipeline(OglModule& mod, const std::string& name, bool hasGeom) {
		fggl::gfx::ShaderConfig config;
		config.name = name;
		config.vertex = name+"_vert.glsl";
		config.fragment = name+"_frag.glsl";
		if ( hasGeom ) {
			config.geometry = name+"_geom.glsl";
		}
		mod->cache.load(config);
	}

	inline void loadBasicPipeline(OglModule& mod, const std::string &name) {
		loadPipeline(mod, name, false);
	}

	//
	// fake module/callbacks - our ECS doesn't have module/callback support yet.
	// 
	void onStaticMeshAdded(ecs3::World& ecs, ecs::entity_t entity, OglModule& mod) {
        //spdlog::info("[CALLBACK] static mesh added, renderable?");
        /*
		auto meshData = ecs.get<gfx::StaticMesh>(entity);
		auto pipeline = mod->cache.get(meshData->pipeline);

		auto glMesh = mod->renderer.upload(meshData->mesh);
		glMesh.pipeline = pipeline;

		ecs.set<fggl::gfx::GlRenderToken>(entity, &glMesh);
         */
	}

	inline void renderMeshes(OglModule& mod, ecs3::World& ecs, float dt) {
		// get the camera
		auto cameras = ecs.findMatching<fggl::gfx::Camera>();
		if ( cameras.empty() ) {
			return;
		}
		auto camera = cameras[0];

		// get the models
        mod->renderer.render(ecs, camera, dt);
	}

}


#endif
