
#ifndef FGGL_GFX_WINDOWING_H
#define FGGL_GFX_WINDOWING_H

#include <memory>
#include <fggl/gfx/paint.hpp>

namespace fggl::gfx {

    class Graphics {
        public:
            virtual ~Graphics() = default;
            virtual void clear() = 0;

            virtual void draw2D(const Paint& paint) = 0;
    };

    class Window {
        public:
            virtual ~Window() = default;
            virtual void activate() const = 0;

            template<typename T>
            void make_graphics() {
                activate();
                m_graphics = std::make_unique<T>();
            }

            virtual void frameStart() = 0;
            virtual void frameEnd() = 0;

            Graphics& graphics() {
                return *m_graphics;
            }

        protected:
            std::unique_ptr<Graphics> m_graphics;
    };

    class WindowService {
        public:
            Window& create();
    };

} // namespace fggl::gfx

#endif
