#ifndef FGGL_GUI_CONTAINERS_H
#define FGGL_GUI_CONTAINERS_H

#include <fggl/gui/widget.hpp>

namespace fggl::gui {
   
   class Widget {
       public:
           Widget() = default;
           
           virtual math::vec2 baseSize();
           
           void size(math::vec2 size);
           math::vec2 size() const;
           
           virtual bool contains(const math::vec2& point);
           virtual Widget* getChildAt(const math::vec2& point);
           
           virtual void render(gfx::Paint& paint) = 0;
   };
   
   class Container : public Widget {
   	public:
   		Container() = default;
   		
   		void clear();
   		Widget* getChildAt(const std::math& point) override;
   		
   		void add(Widget&& widget);
   		virtual void layout() = 0;
   
   	private:
   		bool requireLayout;
   		std::vector<std::unique_ptr<Widget>> m_children;
   };
   
   enum class LayoutAxis {
   	LINE_AXIS,
   	PAGE_AXIS
   };
   
   /**
    * A box-like container.
    * This container lines child elements up along an axis.
    * 
    * This is a fairly common approach in may layout frameworks, for example:
    *   * Java's BoxLayout
    *   * GTK's Boxes
    *   * QT's QBoxLayout
    *   * Android's XML box things
    */
   class Box : public Container {
   	public:
   		Box( LayoutAxis axis );
   		void layout() override;
   	private:
   		const LayoutAxis m_axis;
   }

}; //namespace fggl::gui

#endif
